@@ -1,7 +1,7 @@
 /*************************************************************************************************
  * Raving Bulletin Board System
  *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
- * This file is part of RBBS, a personal full-text search system.
+ * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
  * License, or any later version.
@@ -32,6 +32,10 @@
 #define DUMPNAME    "rbbsdump.xml"       /* saving filename of the dump */
 #define DUMPDBCMD   "dumpdb"             /* command name to dump all articles */
 #define DELEDBCMD   "deledb"             /* command name to delete the database */
+#define CRPREFIX    "crypt:"             /* prefix for password of criptgraph */
+#define TOPICCHAR   '!'                  /* beginning character for topic */
+#define ITEMCHAR    '*'                  /* beginning character for list item */
+#define ASISCHAR    '|'                  /* beginning character for asis text */
 #define SINKPREFIX  SINKSTR ":"          /* prefix for sinking responses */
 
 enum {                                   /* enumeration for actions */
@@ -49,9 +53,9 @@ enum {                                   /* enumeration for actions */
 };
 
 enum {                                   /* enumeration for focusing article */
-  FCSNORMAL,                             /* show the body and a part of responses */
-  FCSDETAIL,                             /* show the body and all responses */
-  FCSSIMPLE                              /* show a part of responses */
+  FCSNORMAL,                             /* show whole of the body and a part of responses */
+  FCSDETAIL,                             /* show whole of the body and all responses */
+  FCSSIMPLE                              /* show the form to post a response */
 };
 
 
@@ -60,14 +64,18 @@ const char *g_systemuri = NULL;          /* URI of this system */
 const char *g_dbname = NULL;             /* name of the database */
 const char *g_helpfile = NULL;           /* path of the help file */
 const char *g_stylefile = NULL;          /* path of the style file */
+int g_jetlag = DEFJETLAG;                /* jet lag */
 const char *g_pagelang = NULL;           /* language of the page */
 const char *g_pagetitle = NULL;          /* title of the page */
 const char *g_pageauthor = NULL;         /* author of the page */
 const char *g_pagesummary = NULL;        /* summary of the page */
 int g_norobot = FALSE;                   /* whether to forbid robots */
+const char *g_captionfile = NULL;        /* path of the caption file */
 const char *g_homepageuri = NULL;        /* URI of the home page */
 const char *g_adminpass = NULL;          /* password for administration */
 const char *g_writepass = NULL;          /* password for writing */
+int g_httpauth = FALSE;                  /* whether to use basic authentication */
+int g_postauth = FALSE;                  /* whether to authorize by each article */
 const char *g_artflags = NULL;           /* default flags for a new article */
 const char *g_resflags = NULL;           /* default flags for a new response */
 const char *g_languages = NULL;          /* list of supported languages */
@@ -89,6 +97,7 @@ const char *g_prevlabel = NULL;          /* label to go to the previous page */
 const char *g_nextlabel = NULL;          /* label to go to the next page */
 const char *g_editlabel = NULL;          /* label to edit the existing */
 const char *g_focuslabel = NULL;         /* label to focus the page or an article */
+const char *g_responselabel = NULL;      /* label to write a response */
 const char *g_passwordlabel = NULL;      /* label for the password */
 const char *g_upstamplabel = NULL;       /* label for the update the time stamp */
 const char *g_idlabel = NULL;            /* label for the ID string of an article */
@@ -109,28 +118,32 @@ const char *g_backlabel = NULL;          /* label for general back button */
 const char *g_anykindlabel = NULL;       /* label for selection of any kind */
 
 
-/* global variables for parameters and cookies */
+/* global variables for parameters, cookies, and authentication */
+const char *p_userpass = NULL;           /* password for administration or writing */
 int p_action = ACTVIEW;                  /* number of the action */
 int p_page = 0;                          /* number of skipping pages */
 int p_focus = FCSNORMAL;                 /* number of the focusing mode */
 int p_sort = -1;                         /* sorting order of articles */
 int p_upstamp = FALSE;                   /* whether to update the modification time */
 const char *p_id = NULL;                 /* ID of an article */
-const char *p_password = NULL;           /* password for administration or writing */
 const char *p_language = NULL;           /* language of an article */
 const char *p_category = NULL;           /* category of an article */
 const char *p_subject = NULL;            /* subject of an article */
 const char *p_author = NULL;             /* author of an article */
 const char *p_text = NULL;               /* body text of an article */
+const char *p_password = NULL;           /* password of an article */
+const char *p_hash = NULL;               /* hash code of an article */
 const char *p_feed = NULL;               /* ID of an article for Atom feed */
 const char *c_author = NULL;             /* author in the cookie */
+const char *a_author = NULL;             /* author via basic authentication */
 
 
 /* other global variables */
 RBBS *g_rbbs = NULL;                     /* database handler */
 const char *g_curdate = NULL;            /* current date */
-int g_rmpost = FALSE;                     /* whether the request method is POST */
+int g_rmpost = FALSE;                    /* whether the request method is POST */
 int g_cannext = FALSE;                   /* whether the next page can be shown */
+const char *g_rtryres = "";              /* text for re-try to post a response */
 int g_tabidx = 0;                        /* counter for tab indexes */
 
 
@@ -147,7 +160,8 @@ void sendfeed(void);
 void sendhtml(void);
 int canadmin(void);
 int canwrite(void);
-int authorize(int admin);
+int canartadmin(void);
+int authorize(void);
 void shownote(const char *msg);
 void showpagenavi(void);
 void showcaption(void);
@@ -159,7 +173,7 @@ void showartwithctl(ARTICLE *art);
 void showartform(void);
 void showartconfirm(void);
 void showartformback(void);
-void asiscat(CBDATUM *buf, const char *text);
+void plaincat(CBDATUM *buf, const char *text, int trim);
 void mypostarticle(void);
 void mydeletearticle(void);
 int mypostresponse(void);
@@ -193,6 +207,8 @@ int main(int argc, char **argv){
       g_helpfile = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "stylefile:")){
       g_stylefile = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "jetlag:")){
+      g_jetlag = atoi(skiplabel(tmp));
     } else if(cbstrfwimatch(tmp, "pagelang:")){
       g_pagelang = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "pagetitle:")){
@@ -203,12 +219,18 @@ int main(int argc, char **argv){
       g_pagesummary = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "norobot:")){
       g_norobot = !strcmp(skiplabel(tmp), "true");
+    } else if(cbstrfwimatch(tmp, "captionfile:")){
+      g_captionfile = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "homepageuri:")){
       g_homepageuri = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "adminpass:")){
       g_adminpass = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "writepass:")){
       g_writepass = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "httpauth:")){
+      g_httpauth = !cbstricmp(skiplabel(tmp), "true");
+    } else if(cbstrfwimatch(tmp, "postauth:")){
+      g_postauth = !cbstricmp(skiplabel(tmp), "true");
     } else if(cbstrfwimatch(tmp, "artflags:")){
       g_artflags = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "resflags:")){
@@ -260,6 +282,8 @@ int main(int argc, char **argv){
       g_editlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "focuslabel:")){
       g_focuslabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "responselabel:")){
+      g_responselabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "passwordlabel:")){
       g_passwordlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "upstamplabel:")){
@@ -302,10 +326,12 @@ int main(int argc, char **argv){
   if(!g_dbname) senderror("dbname is undefined.");
   if(!g_helpfile) senderror("helpfile is undefined.");
   if(!g_stylefile) senderror("stylefile is undefined.");
+  jetlag(g_jetlag);
   if(!g_pagelang) senderror("pagelang is undefined.");
   if(!g_pagetitle) senderror("pagetitle is undefined.");
   if(!g_pageauthor) senderror("pageauthor is undefined.");
   if(!g_pagesummary) senderror("pagesummary is undefined.");
+  if(!g_captionfile) g_captionfile = "";
   if(!g_homepageuri) senderror("homepageuri is undefined.");
   if(!g_adminpass) senderror("adminpass is undefined.");
   if(!g_writepass) senderror("writepass is undefined.");
@@ -328,6 +354,7 @@ int main(int argc, char **argv){
   if(!g_nextlabel) senderror("nextlabel is undefined.");
   if(!g_editlabel) senderror("editlabel is undefined.");
   if(!g_focuslabel) senderror("focuslabel is undefined.");
+  if(!g_responselabel) senderror("responselabel is undefined.");
   if(!g_passwordlabel) senderror("passwordlabel is undefined");
   if(!g_upstamplabel) senderror("upstamplabel is undefined");
   if(!g_idlabel) senderror("idlabel is undefined");
@@ -349,6 +376,7 @@ int main(int argc, char **argv){
   /* read parameters */
   params = getparameters();
   cbglobalgc(params, (void (*)(void *))cbmapclose);
+  if((tmp = cbmapget(params, "userpass", -1, NULL)) != NULL) p_userpass = tmp;
   if((tmp = cbmapget(params, "action", -1, NULL)) != NULL){
     if(!strcmp(tmp, DUMPDBCMD)){
       p_action = ACTDUMPDB;
@@ -363,30 +391,36 @@ int main(int argc, char **argv){
   if((tmp = cbmapget(params, "sort", -1, NULL)) != NULL) p_sort = atoi(tmp);
   if((tmp = cbmapget(params, "upstamp", -1, NULL)) != NULL) p_upstamp = atoi(tmp);
   if((tmp = cbmapget(params, "id", -1, NULL)) != NULL) p_id = tmp;
-  if((tmp = cbmapget(params, "password", -1, NULL)) != NULL) p_password = tmp;
   if((tmp = cbmapget(params, "language", -1, NULL)) != NULL) p_language = tmp;
   if((tmp = cbmapget(params, "category", -1, NULL)) != NULL) p_category = tmp;
   if((tmp = cbmapget(params, "subject", -1, NULL)) != NULL) p_subject = tmp;
   if((tmp = cbmapget(params, "author", -1, NULL)) != NULL) p_author = tmp;
   if((tmp = cbmapget(params, "text", -1, NULL)) != NULL) p_text = tmp;
+  if((tmp = cbmapget(params, "password", -1, NULL)) != NULL) p_password = tmp;
+  if((tmp = cbmapget(params, "hash", -1, NULL)) != NULL) p_hash = tmp;
   if((tmp = cbmapget(params, "feed", -1, NULL)) != NULL) p_feed = tmp;
+  if(!p_userpass) p_userpass = "";
   if(p_action < ACTVIEW || p_action > ACTHELP) p_action = ACTVIEW;
   if(p_page < 0) p_page = 0;
   if(p_focus < FCSNORMAL || p_focus > FCSSIMPLE) p_focus = FCSNORMAL;
   if(p_sort < SORTCDATEA || p_sort >  SORTMDATED) p_sort = g_sort;
   if(!p_id) p_id = "";
-  if(!p_password) p_password = "";
   if(!p_language) p_language = "";
   if(!p_category) p_category = "";
   if(!p_subject) p_subject = "";
   if(!p_author) p_author = "";
   if(!p_text) p_text = "";
+  if(!p_password) p_password = "";
+  if(!p_hash) p_hash = "";
   if(!p_feed) p_feed = "";
   /* read cookies */
   cooks = getcookies();
   cbglobalgc(cooks, (void (*)(void *))cbmapclose);
   if((tmp = cbmapget(cooks, "author", -1, NULL)) != NULL) c_author = tmp;
   if(!c_author) c_author = "";
+  /* read authentication */
+  a_author = "";
+  if(g_httpauth && (tmp = getenv("REMOTE_USER")) != NULL) a_author = tmp;
   /* open the database */
   switch(p_action){
   case ACTVIEW:
@@ -401,7 +435,7 @@ int main(int argc, char **argv){
     g_rbbs = rbbsopen(g_dbname, g_rmpost);
     break;
   case ACTPOSTRES:
-    g_rbbs = rbbsopen(g_dbname, g_rmpost && p_author[0] != '\0' && p_text != '\0');
+    g_rbbs = rbbsopen(g_dbname, g_rmpost && p_text != '\0');
     break;
   case ACTDUMPDB:
     g_rbbs = rbbsopen(g_dbname, FALSE);
@@ -701,6 +735,7 @@ void sendfeed(void){
 
 /* send HTML page */
 void sendhtml(void){
+  const char *fstr;
   char *tmp;
   printf("Cache-Control: no-cache, must-revalidate\r\n");
   printf("Pragma: no-cache\r\n");
@@ -739,7 +774,15 @@ void sendhtml(void){
   xprintf("<meta name=\"author\" content=\"%@\" />\n", g_pageauthor);
   xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
   if(g_norobot) xprintf("<meta name=\"robots\" content=\"NOINDEX,NOFOLLOW\" />\n");
+  tmp = dateforhttp(g_curdate, TRUE);
+  xprintf("<meta name=\"localtime\" content=\"%@\" />\n", tmp);
+  free(tmp);
   xprintf("<link rel=\"contents\" href=\"%@?action=%d\" />\n", g_systemuri, ACTLIST);
+  fstr = FEEDCDATE;
+  if(g_sort == SORTMDATEA || g_sort == SORTMDATED) fstr = FEEDMDATE;
+  if(p_id[0] != '\0') fstr = p_id;
+  xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@?feed=%@\" title=\"Atom\" />\n",
+          ATOMTYPE, g_systemuri, fstr);
   if((tmp = cbreadfile(g_stylefile, NULL)) != NULL){
     xprintf("<style type=\"%@\">\n", CSSTYPE);
     xprintf("%s", tmp);
@@ -756,44 +799,50 @@ void sendhtml(void){
   }
   showpagenavi();
   xprintf("<hr />\n");
-  switch(p_action){
-  case ACTVIEW:
-    showarticles();
-    break;
-  case ACTLIST:
-    showlist();
-    break;
-  case ACTSEARCH:
-    showsearchform();
-    showarticles();
-    break;
-  case ACTARTFORM:
-    if(authorize(p_id[0] != '\0')) showartform();
-    break;
-  case ACTARTCNFM:
-    if(authorize(p_id[0] != '\0')) showartconfirm();
-    break;
-  case ACTPOSTART:
-    if(authorize(p_id[0] != '\0')) mypostarticle();
-    break;
-  case ACTDELEART:
-    if(authorize(p_id[0] != '\0')) mydeletearticle();
-    break;
-  case ACTPOSTRES:
-    if(mypostresponse()) showarticles();
-    break;
-  case ACTDELEDB:
-    if(authorize(TRUE)) deletedb();
-    break;
-  case ACTHELP:
-    showhelp();
-    break;
-  default:
-    break;
+  if(authorize()){
+    switch(p_action){
+    case ACTVIEW:
+      showarticles();
+      break;
+    case ACTLIST:
+      showlist();
+      break;
+    case ACTSEARCH:
+      showsearchform();
+      showarticles();
+      break;
+    case ACTARTFORM:
+      showartform();
+      break;
+    case ACTARTCNFM:
+      showartconfirm();
+      break;
+    case ACTPOSTART:
+      mypostarticle();
+      break;
+    case ACTDELEART:
+      mydeletearticle();
+      break;
+    case ACTPOSTRES:
+      if(mypostresponse()) showarticles();
+      break;
+    case ACTDELEDB:
+      deletedb();
+      break;
+    case ACTHELP:
+      showhelp();
+      break;
+    default:
+      break;
+    }
   }
   showpagenavi();
-  if((p_action == ACTVIEW || p_action == ACTSEARCH) && p_id[0] == '\0')
-    xprintf("<div class=\"sysversion\">Powered by RBBS %@.</div>\n", RBBS_VERSION);
+  xprintf("<div class=\"sysinfo\">\n");
+  tmp = dateforhttp(g_curdate, TRUE);
+  xprintf("<div class=\"syscurdate\">%@</div>\n", tmp);
+  free(tmp);
+  xprintf("<div class=\"sysversion\">Powered by RBBS %@.</div>\n", RBBS_VERSION);
+  xprintf("</div>\n");
   xprintf("</body>\n");
   xprintf("</html>\n");
 }
@@ -802,7 +851,9 @@ void sendhtml(void){
 /* check whether the user can administrate the system */
 int canadmin(void){
   if(g_adminpass[0] == '\0') return TRUE;
-  if(!strcmp(p_password, g_adminpass)) return TRUE;
+  if(cbstrfwmatch(g_adminpass, CRPREFIX))
+    return matchcrypt(p_userpass, g_adminpass + strlen(CRPREFIX));
+  if(!strcmp(p_userpass, g_adminpass)) return TRUE;
   return FALSE;
 }
 
@@ -811,20 +862,61 @@ int canadmin(void){
 int canwrite(void){
   if(canadmin()) return TRUE;
   if(g_writepass[0] == '\0') return TRUE;
-  if(!strcmp(p_password, g_writepass)) return TRUE;
+  if(cbstrfwmatch(g_writepass, CRPREFIX))
+    return matchcrypt(p_userpass, g_writepass + strlen(CRPREFIX));
+  if(!strcmp(p_userpass, g_writepass)) return TRUE;
   return FALSE;
 }
 
 
+/* check whether the user can administrate the article */
+int canartadmin(void){
+  ARTICLE *art;
+  int match;
+  if(!g_rbbs || !(art = getarticle(g_rbbs, p_id))) return FALSE;
+  match = g_postauth && matchcrypt(p_userpass, artpassword(art));
+  freearticle(art);
+  return match;
+}
+
+
 /* check the password and show the form to input a password */
-int authorize(int admin){
+int authorize(void){
+  int admin, artadm;
+  admin = FALSE;
+  artadm = FALSE;
+  switch(p_action){
+  case ACTVIEW:
+  case ACTLIST:
+  case ACTSEARCH:
+  case ACTHELP:
+  case ACTPOSTRES:
+    return TRUE;
+  case ACTARTFORM:
+  case ACTARTCNFM:
+  case ACTPOSTART:
+  case ACTDELEART:
+    if(p_id[0] != '\0'){
+      admin = TRUE;
+      artadm = TRUE;
+    }
+    break;
+  case ACTDELEDB:
+    admin = TRUE;
+    break;
+  default:
+    break;
+  }
   if(admin){
     if(canadmin()) return TRUE;
+    if(artadm && canartadmin()) return TRUE;
   } else {
     if(canwrite()) return TRUE;
   }
-  if(p_password[0] != '\0'){
+  if(p_userpass[0] != '\0'){
     shownote("The password did not match.  Try again.");
+  } else if(artadm && g_postauth){
+    shownote("The password of the article is needed.");
   } else if(admin){
     shownote("The password of the administrator is needed.");
   } else {
@@ -836,7 +928,7 @@ int authorize(int admin){
   xprintf("<tr>\n");
   xprintf("<td class=\"tlabel\">%@:</td>\n", g_passwordlabel);
   xprintf("<td class=\"tbody\">\n");
-  xprintf("<input type=\"password\" name=\"password\" value=\"\" size=\"16\""
+  xprintf("<input type=\"password\" name=\"userpass\" value=\"\" size=\"16\""
           " class=\"password\" tabindex=\"%d\" />\n", ++g_tabidx);
   xprintf("</td>\n");
   xprintf("</tr>\n");
@@ -889,7 +981,7 @@ void showpagenavi(void){
   xprintf("<a href=\"%@?action=%d\" class=\"navianc\" tabindex=\"%d\" accesskey=\"9\">%@</a>\n",
           g_systemuri, ACTARTFORM, ++g_tabidx, g_postlabel);
   fstr = FEEDCDATE;
-  if(p_sort == SORTMDATEA || p_sort == SORTMDATED) fstr = FEEDMDATE;
+  if(g_sort == SORTMDATEA || g_sort == SORTMDATED) fstr = FEEDMDATE;
   xprintf("<a href=\"%@?feed=%@\" type=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, fstr, ATOMTYPE, ++g_tabidx, g_feedlabel);
   xprintf("<a href=\"%@?action=%d\" rel=\"top\" class=\"navianc\""
@@ -919,9 +1011,15 @@ void showpagenavi(void){
 
 /* show the caption of the page */
 void showcaption(void){
+  char *buf;
   xprintf("<div class=\"pagecaption\">\n");
-  xprintf("<h1>%@</h1>\n", g_pagetitle);
-  xprintf("<p>%@</p>\n", g_pagesummary);
+  if(g_captionfile[0] != '\0' && (buf = cbreadfile(g_captionfile, NULL)) != NULL){
+    xprintf("%s", buf);
+    free(buf);
+  } else {
+    xprintf("<h1>%@</h1>\n", g_pagetitle);
+    xprintf("<p>%@</p>\n", g_pagesummary);
+  }
   xprintf("</div>\n");
   xprintf("<hr />\n");
 }
@@ -987,7 +1085,7 @@ void showlist(void){
     }
     free(id);
   }
-  if(snum < 1) xprintf("<span class=\"note\">There is no corresponding article.</span>\n");
+  if(snum < 1) shownote("There is no corresponding article.");
   xprintf("</div>\n");
   xprintf("<hr />\n");
   if(snum > g_shownum) g_cannext = TRUE;
@@ -1137,11 +1235,11 @@ void showarticles(void){
 /* show one article with the navigation control */
 void showartwithctl(ARTICLE *art){
   static int anum = 0;
-  char akey[NUMBUFSIZ], *html;
+  char buf[NUMBUFSIZ], *html;
   if(++anum < 9){
-    sprintf(akey, "%d", anum);
+    sprintf(buf, "%d", anum);
   } else {
-    sprintf(akey, "#");
+    sprintf(buf, "#");
   }
   xprintf("<div class=\"artctl\">\n");
   xprintf("<div class=\"artnavi\">\n");
@@ -1149,23 +1247,36 @@ void showartwithctl(ARTICLE *art){
           g_systemuri, ACTARTFORM, artid(art), ++g_tabidx, g_editlabel);
   xprintf("<a href=\"%@?feed=%@\" type=\"%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, artid(art), ATOMTYPE, ++g_tabidx, g_feedlabel);
-  xprintf("<a href=\"%@?id=%@&amp;focus=%d\" class=\"navianc\" tabindex=\"%d\" accesskey=\"%s\">"
-          "%@</a>\n", g_systemuri, artid(art), FCSDETAIL, ++g_tabidx, akey, g_focuslabel);
+  xprintf("<a href=\"%@?id=%@&amp;focus=%d\" class=\"navianc\" tabindex=\"%d\">"
+          "%@</a>\n", g_systemuri, artid(art), FCSDETAIL, ++g_tabidx, g_focuslabel);
+  if(!strchr(artflags(art), LONECHAR)){
+    xprintf("<a href=\"%@?id=%@&amp;focus=%d#%@R\" class=\"navianc\""
+            " tabindex=\"%d\" accesskey=\"%s\">%@</a>\n",
+            g_systemuri, artid(art), FCSSIMPLE, artid(art), ++g_tabidx, buf, g_responselabel);
+  } else {
+    xprintf("<span class=\"navivoid\">%@</span>\n", g_responselabel);
+  }
   xprintf("</div>\n");
   html = arttohtml(art, p_focus != FCSSIMPLE ? 0 : g_blknum, p_focus == FCSDETAIL ? 0 : g_resnum);
   xprintf("%s", html);
   free(html);
-  if(!strchr(artflags(art), LONECHAR) && p_id[0] != '\0'){
-    xprintf("<form method=\"post\" action=\"%@#%@T\">\n", g_systemuri, artid(art));
+  if(p_focus == FCSSIMPLE && !strchr(artflags(art), LONECHAR)){
+    if(g_rtryres[0] != '\0') shownote("Some responses are crossing.  Try again.");
+    xprintf("<form method=\"post\" action=\"%@#%@R\" id=\"%@R\">\n",
+            g_systemuri, artid(art), artid(art));
     xprintf("<div class=\"resform\">\n");
     xprintf("<span class=\"formitem\">\n");
-    xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"8\""
-            " class=\"text\" tabindex=\"%d\" />\n",
-            p_author[0] != '\0' ? p_author : c_author, ++g_tabidx);
+    if(a_author[0] != '\0'){
+      xprintf("<span class=\"resauthor\">%@</span>\n", a_author);
+    } else {
+      xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"8\""
+              " class=\"text\" tabindex=\"%d\" />\n",
+              p_author[0] != '\0' ? p_author : c_author, ++g_tabidx);
+    }
     xprintf("</span>\n");
     xprintf("<span class=\"formitem\">\n");
-    xprintf("<input type=\"text\" name=\"text\" value=\"\" size=\"80\""
-            " class=\"text\" tabindex=\"%d\" />\n", ++g_tabidx);
+    xprintf("<input type=\"text\" name=\"text\" value=\"%@\" size=\"80\""
+            " class=\"text\" tabindex=\"%d\" />\n", g_rtryres, ++g_tabidx);
     xprintf("</span>\n");
     xprintf("<span class=\"formitem\">\n");
     xprintf("<input type=\"submit\" value=\"%@\" class=\"submit\""
@@ -1175,6 +1286,7 @@ void showartwithctl(ARTICLE *art){
     xprintf("<div class=\"hidden\">\n");
     xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTPOSTRES);
     xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", artid(art));
+    xprintf("<input type=\"hidden\" name=\"hash\" value=\"%@\" />\n", arthash(art));
     xprintf("<input type=\"hidden\" name=\"focus\" value=\"%d\" />\n", FCSSIMPLE);
     xprintf("</div>\n");
     xprintf("</form>\n");
@@ -1260,8 +1372,12 @@ void showartform(void){
     xprintf("<tr>\n");
     xprintf("<td class=\"tlabel\">%@:</td>\n", g_authorlabel);
     xprintf("<td class=\"tbody\">\n");
-    xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"16\""
-            " class=\"text\" tabindex=\"%d\" />\n", p_author, ++g_tabidx);
+    if(a_author[0] != '\0'){
+      xprintf("%@\n", a_author);
+    } else {
+      xprintf("<input type=\"text\" name=\"author\" value=\"%@\" size=\"16\""
+              " class=\"text\" tabindex=\"%d\" />\n", p_author, ++g_tabidx);
+    }
     xprintf("</td>\n");
     xprintf("</tr>\n");
     xprintf("<tr>\n");
@@ -1273,6 +1389,15 @@ void showartform(void){
     free(xml);
     xprintf("</td>\n");
     xprintf("</tr>\n");
+    if(g_postauth){
+      xprintf("<tr>\n");
+      xprintf("<td class=\"tlabel\">%@:</td>\n", g_passwordlabel);
+      xprintf("<td class=\"tbody\">\n");
+      xprintf("<input type=\"password\" name=\"password\" value=\"%@\" size=\"16\""
+              " class=\"text\" tabindex=\"%d\" />\n", p_password, ++g_tabidx);
+      xprintf("</td>\n");
+      xprintf("</tr>\n");
+    }
   }
   xprintf("<tr>\n");
   xprintf("<td class=\"tlabel\"></td>\n");
@@ -1286,9 +1411,9 @@ void showartform(void){
   xprintf("</table>\n");
   xprintf("</div>\n");
   xprintf("<div class=\"hidden\">\n");
+  xprintf("<input type=\"hidden\" name=\"userpass\" value=\"%@\" />\n", p_userpass);
   xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTARTCNFM);
   xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", p_id);
-  xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
   xprintf("</div>\n");
   xprintf("</form>\n");
   xprintf("<hr />\n");
@@ -1299,28 +1424,58 @@ void showartform(void){
 void showartconfirm(void){
   ARTICLE *art;
   CBDATUM *buf;
+  CBLIST *lines;
+  const char *pline, *cline, *nline;
   char *xml, *tmp;
-  int len;
+  int i, len;
   buf = cbdatumopen("", 0);
   if(p_id[0] != '\0'){
     xml = strnormalize(p_text, FALSE, TRUE);
     xsprintf(buf, "%s", xml);
     free(xml);
   } else {
-    xsprintf(buf, "<article>\n");
+    xsprintf(buf, "<article password=\"%@\">\n", makecrypt(p_password));
     xsprintf(buf, "<head>\n");
     xsprintf(buf, "<flags>%@</flags>\n", g_artflags);
     xsprintf(buf, "<language>%@</language>\n", p_language);
     xsprintf(buf, "<category>%@</category>\n", p_category);
     xsprintf(buf, "<subject>%@</subject>\n", p_subject);
-    xsprintf(buf, "<author>%@</author>\n", p_author);
+    xsprintf(buf, "<author>%@</author>\n", a_author[0] != '\0' ? a_author : p_author);
     xsprintf(buf, "<cdate>%@</cdate>\n", g_curdate);
     xsprintf(buf, "<mdate>%@</mdate>\n", g_curdate);
     xsprintf(buf, "</head>\n");
     xsprintf(buf, "<body>\n");
-    xml = strnormalize(p_text, FALSE, TRUE);
+    xml = strnormalize(p_text, FALSE, FALSE);
     if(xml[0] != '<'){
-      asiscat(buf, xml);
+      lines = cbsplit(xml, -1, "\n");
+      for(i = 0; i < cblistnum(lines); i++){
+        pline = i > 0 ? cblistval(lines, i - 1, NULL) : "";
+        cline = cblistval(lines, i, NULL);
+        nline = i < cblistnum(lines) - 1 ? cblistval(lines, i + 1, NULL) : "";
+        if(cline[0] != '\0'){
+          if(cline[0] == TOPICCHAR){
+            xsprintf(buf, "<topic>");
+            plaincat(buf, cline + 1, TRUE);
+            xsprintf(buf, "</topic>\n");
+          } else if(cline[0] == ITEMCHAR){
+            if(pline[0] != ITEMCHAR) xsprintf(buf, "<list>\n");
+            xsprintf(buf, "<item>");
+            plaincat(buf, cline + 1, TRUE);
+            xsprintf(buf, "</item>\n");
+            if(nline[0] != ITEMCHAR) xsprintf(buf, "</list>\n");
+          } else if(cline[0] == ASISCHAR){
+            if(pline[0] != ASISCHAR) xsprintf(buf, "<asis>");
+            plaincat(buf, cline + 1, FALSE);
+            xsprintf(buf, "\n");
+            if(nline[0] != ASISCHAR) xsprintf(buf, "</asis>\n");
+          } else {
+            xsprintf(buf, "<para>");
+            plaincat(buf, cline, TRUE);
+            xsprintf(buf, "</para>\n");
+          }
+        }
+      }
+      cblistclose(lines);
     } else {
       xsprintf(buf, "%s", xml);
     }
@@ -1350,15 +1505,17 @@ void showartconfirm(void){
     xprintf("</table>\n");
     xprintf("</div>\n");
     xprintf("<div class=\"hidden\">\n");
+    xprintf("<input type=\"hidden\" name=\"userpass\" value=\"%@\" />\n", p_userpass);
     xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTDELEART);
     xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", p_id);
-    xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
     xprintf("</div>\n");
     xprintf("</form>\n");
   } else if(!articleisok(art)){
     shownote(artemsg(art));
   } else {
     shownote("Confirm the content then submit it.");
+    if(g_postauth && p_id[0] == '\0' && p_password[0] == '\0')
+      shownote("Warning: The password is empty and everyone can edit the article.");
     xml = arttoxml(art);
     xprintf("<div class=\"artctl\">\n");
     tmp = arttohtml(art, 0, 0);
@@ -1398,9 +1555,9 @@ void showartconfirm(void){
     xprintf("</table>\n");
     xprintf("</div>\n");
     xprintf("<div class=\"hidden\">\n");
+    xprintf("<input type=\"hidden\" name=\"userpass\" value=\"%@\" />\n", p_userpass);
     xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTPOSTART);
     xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", p_id);
-    xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
     xprintf("<input type=\"hidden\" name=\"author\" value=\"%@\" />\n", p_author);
     tmp = cbbaseencode(xml, -1);
     xprintf("<input type=\"hidden\" name=\"text\" value=\"%@\" />\n", tmp);
@@ -1424,6 +1581,7 @@ void showartformback(void){
           " tabindex=\"%d\" />\n", g_backlabel, ++g_tabidx);
   xprintf("</div>\n");
   xprintf("<div class=\"hidden\">\n");
+  xprintf("<input type=\"hidden\" name=\"userpass\" value=\"%@\" />\n", p_userpass);
   xprintf("<input type=\"hidden\" name=\"action\" value=\"%d\" />\n", ACTARTFORM);
   xprintf("<input type=\"hidden\" name=\"id\" value=\"%@\" />\n", p_id);
   xprintf("<input type=\"hidden\" name=\"password\" value=\"%@\" />\n", p_password);
@@ -1439,42 +1597,71 @@ void showartformback(void){
 }
 
 
-/* concatenate an as-is text to an XML buffer */
-void asiscat(CBDATUM *buf, const char *text){
+/* concatenate a plain text to an XML buffer */
+void plaincat(CBDATUM *buf, const char *text, int trim){
   const char *prefs[] = {
     "http://", "https://", "ftp://", "gopher://", "ldap://", "file://", NULL
   };
-  const char *pref, *rp;
-  char *tmp;
+  const char *pref, *pv, *rp;
+  char *rtext, *tmp, *mp;
   int i, j;
-  xsprintf(buf, "<asis>");
-  for(i = 0; text[i] != '\0'; i++){
-    pref = NULL;
-    for(j = 0; prefs[j]; j++){
-      if(cbstrfwimatch(text + i, prefs[j])){
-        pref = prefs[j];
+  rtext = cbmemdup(text, -1);
+  if(trim) cbstrtrim(rtext);
+  for(i = 0; rtext[i] != '\0'; i++){
+    if(cbstrfwmatch(rtext + i, "[[") && (pv = strstr(rtext + i, "]]")) != NULL){
+      tmp = cbmemdup(rtext + i + 2, pv - rtext - i - 2);
+      switch(tmp[0]){
+      case '^':
+        xsprintf(buf, "<emp>%@</emp>", tmp + 1);
+        break;
+      case '~':
+        xsprintf(buf, "<cite>%@</cite>", tmp + 1);
+        break;
+      case '+':
+        xsprintf(buf, "<ins>%@</ins>", tmp + 1);
+        break;
+      case '-':
+        xsprintf(buf, "<del>%@</del>", tmp + 1);
+        break;
+      default:
+        if((mp = strchr(tmp, '|')) != NULL){
+          *mp = '\0';
+          xsprintf(buf, "<link to=\"%@\">%@</link>", mp + 1, tmp);
+        } else {
+          xsprintf(buf, "%@", tmp);
+        }
+        break;
       }
-    }
-    if(pref){
-      rp = text + i;
-      while(*rp != '\0' && ((*rp >= '0' && *rp <= '9') || (*rp >= 'A' && *rp <= 'Z') ||
-                            (*rp >= 'a' && *rp <= 'z') || strchr("-_.!~*'();/?:@&=+$,%#", *rp))){
-        rp++;
-      }
-      tmp = cbmemdup(text + i, rp - text - i);
-      xsprintf(buf, "<link to=\"%@\">%@</link>", tmp, tmp);
       free(tmp);
-      i = rp - text - 1;
+      i = pv - rtext + 1;
     } else {
-      switch(text[i]){
-      case '&': xsprintf(buf, "&amp;"); break;
-      case '<': xsprintf(buf, "&lt;"); break;
-      case '>': xsprintf(buf, "&gt;"); break;
-      default: xsprintf(buf, "%c", text[i]); break;
+      pref = NULL;
+      for(j = 0; prefs[j]; j++){
+        if(cbstrfwimatch(rtext + i, prefs[j])){
+          pref = prefs[j];
+        }
+      }
+      if(pref){
+        rp = rtext + i;
+        while(*rp != '\0' && ((*rp >= '0' && *rp <= '9') || (*rp >= 'A' && *rp <= 'Z') ||
+                              (*rp >= 'a' && *rp <= 'z') || strchr("-_.!~*'();/?:@&=+$,%#", *rp))){
+          rp++;
+        }
+        tmp = cbmemdup(rtext + i, rp - rtext - i);
+        xsprintf(buf, "<link to=\"%@\">%@</link>", tmp, tmp);
+        free(tmp);
+        i = rp - rtext - 1;
+      } else {
+        switch(rtext[i]){
+        case '&': xsprintf(buf, "&amp;"); break;
+        case '<': xsprintf(buf, "&lt;"); break;
+        case '>': xsprintf(buf, "&gt;"); break;
+        default: xsprintf(buf, "%c", rtext[i]); break;
+        }
       }
     }
   }
-  xsprintf(buf, "\n</asis>\n");
+  free(rtext);
 }
 
 
@@ -1496,6 +1683,13 @@ void mypostarticle(void){
   }
   if(!articleisok(art)){
     shownote(artemsg(art));
+  } else if(p_id[0] != '\0' && strcmp(p_id, artid(art))){
+    shownote("The ID can not be changed.");
+  } else if(g_postauth && p_id[0] != '\0' && !canadmin() &&
+            !matchcrypt(p_userpass, artpassword(art))){
+    shownote("The password can not be changed.");
+  } else if(a_author[0] != '\0' && strcmp(artauthor(art), a_author)){
+    shownote("The author is invalid.");
   } else if(!postarticle(g_rbbs, art, p_id[0] != '\0' ? p_id : NULL)){
     shownote(rbbsemsg(g_rbbs));
   } else {
@@ -1540,7 +1734,12 @@ int mypostresponse(void){
     shownote("There is no corresponding article.");
     return FALSE;
   }
-  author = strnormalize(p_author, TRUE, TRUE);
+  if(strcmp(arthash(art), p_hash)){
+    freearticle(art);
+    g_rtryres = p_text;
+    return TRUE;
+  }
+  author = strnormalize(a_author[0] != '\0' ? a_author : p_author, TRUE, TRUE);
   text = strnormalize(p_text, TRUE, TRUE);
   if(author[0] == '\0' || text[0] == '\0'){
     free(text);
