@@ -1,7 +1,7 @@
 /*************************************************************************************************
  * Implementation of common features
  *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
- * This file is part of RBBS, a personal full-text search system.
+ * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
  * License, or any later version.
@@ -20,9 +20,9 @@
 /* private function prototypes */
 static int validxmlent(const char *xml);
 static char *uniqueid(void);
+static int validid(const char *id);
 static char *datestr(time_t t);
 static int validdate(const char *date);
-static int jetlag(void);
 static const char *wdayname(int wday);
 static const char *monname(int mon);
 static int strisprintable(const char *str);
@@ -302,12 +302,13 @@ ARTICLE *makearticle(const char *xml){
   CBLIST *elems, *stack;
   CBMAP *attrs;
   CBDATUM *cont;
-  const char *elem, *name, *pname, *ntext, *celem;
+  const char *elem, *name, *pname, *ntext, *celem, *val;
   char *rtext;
   int i, j, isart, ishead, isbody, istail, bstart, rstart;
   assert(xml);
   art = cbmalloc(sizeof(ARTICLE));
   art->id = NULL;
+  art->password = NULL;
   art->flags = NULL;
   art->language = NULL;
   art->category = NULL;
@@ -318,6 +319,7 @@ ARTICLE *makearticle(const char *xml){
   art->body = cblistopen();
   art->tail = cblistopen();
   art->errmsg = NULL;
+  art->hash = cbsprintf("%08X", dpouterhash(xml, -1));
   rtext = strnormalize(xml, FALSE, FALSE);
   elems = cbxmlbreak(rtext, TRUE);
   free(rtext);
@@ -375,6 +377,10 @@ ARTICLE *makearticle(const char *xml){
             art->errmsg = cbsprintf("Invalid-XML: \"article\" must not be empty");
           } else {
             isart = TRUE;
+            if(!art->id && (val = cbmapget(attrs, "id", -1, NULL)) != NULL)
+              art->id = cbmemdup(val, -1);
+            if(!art->password && (val = cbmapget(attrs, "password", -1, NULL)) != NULL)
+              art->password = cbmemdup(val, -1);
           }
         } else if(!strcmp(name, "head")){
           if(!pname || strcmp(pname, "article")){
@@ -606,7 +612,12 @@ ARTICLE *makearticle(const char *xml){
     }
     if(art->errmsg) break;
   }
-  art->id = uniqueid();
+  if(!art->id){
+    art->id = uniqueid();
+  } else if(!validid(art->id)){
+    if(!art->errmsg) art->errmsg = cbsprintf("Invalid-XML: the ID is invalid");
+  }
+  if(!art->password) art->password = cbmemdup("", 0);
   if(!art->flags) art->flags = cbmemdup("", 0);
   if(!art->language || art->language[0] == '\0'){
     if(!art->errmsg) art->errmsg = cbsprintf("Invalid-XML: the language is undefined");
@@ -646,6 +657,7 @@ ARTICLE *makearticle(const char *xml){
 /* Release resources of an article object. */
 void freearticle(ARTICLE *art){
   assert(art);
+  free(art->hash);
   free(art->errmsg);
   cblistclose(art->tail);
   cblistclose(art->body);
@@ -656,6 +668,7 @@ void freearticle(ARTICLE *art){
   free(art->category);
   free(art->language);
   free(art->flags);
+  free(art->password);
   free(art->id);
   free(art);
 }
@@ -676,6 +689,13 @@ const char *artemsg(ARTICLE *art){
 }
 
 
+/* Get the hash code of an article. */
+const char *arthash(ARTICLE *art){
+  assert(art);
+  return art->hash;
+}
+
+
 /* Get the ID string of an article. */
 const char *artid(ARTICLE *art){
   assert(art);
@@ -683,6 +703,13 @@ const char *artid(ARTICLE *art){
 }
 
 
+/* Get the password of an article. */
+const char *artpassword(ARTICLE *art){
+  assert(art);
+  return art->password;
+}
+
+
 /* Get the flags of an article. */
 const char *artflags(ARTICLE *art){
   assert(art);
@@ -779,7 +806,7 @@ char *arttoxml(ARTICLE *art){
   int i, j, inlist, initem;
   assert(art);
   buf = cbdatumopen("", 0);
-  xsprintf(buf, "<article>\n");
+  xsprintf(buf, "<article id=\"%@\" password=\"%@\">\n", art->id, art->password);
   xsprintf(buf, "<head>\n");
   xsprintf(buf, "<flags>%@</flags>\n", art->flags);
   xsprintf(buf, "<language>%@</language>\n", art->language);
@@ -875,9 +902,10 @@ char *arttoxml(ARTICLE *art){
 char *arttotext(ARTICLE *art){
   CBDATUM *buf;
   CBLIST *elems;
-  const char *blk, *elem;
-  char *rtext;
-  int i, j, fst;
+  CBMAP *attrs;
+  const char *blk, *elem, *name, *tmp;
+  char *flags, *author, *cdate, *text, *rtext;
+  int i, j;
   assert(art);
   buf = cbdatumopen("", 0);
   xsprintf(buf, "%s\n", art->flags);
@@ -891,35 +919,53 @@ char *arttotext(ARTICLE *art){
   for(i = 0; i < cblistnum(art->body); i++){
     blk = cblistval(art->body, i, NULL);
     elems = cbxmlbreak(blk, FALSE);
-    fst = TRUE;
     for(j = 0; j < cblistnum(elems); j++){
       elem = cblistval(elems, j, NULL);
       if(elem[0] == '<') continue;
       rtext = cbxmlunescape(elem);
-      if(!fst) xsprintf(buf, " ");
       xsprintf(buf, "%s", rtext);
       free(rtext);
-      fst = TRUE;
     }
     xsprintf(buf, "\n");
     cblistclose(elems);
   }
   xsprintf(buf, "\n");
   for(i = 0; i < cblistnum(art->tail); i++){
+    flags = NULL;
+    author = NULL;
+    cdate = NULL;
+    text = NULL;
     blk = cblistval(art->tail, i, NULL);
     elems = cbxmlbreak(blk, FALSE);
-    fst = TRUE;
     for(j = 0; j < cblistnum(elems); j++){
       elem = cblistval(elems, j, NULL);
-      if(elem[0] == '<') continue;
-      rtext = cbxmlunescape(elem);
-      if(!fst) xsprintf(buf, " ");
-      xsprintf(buf, "%s", rtext);
-      free(rtext);
-      fst = TRUE;
+      if(elem[0] == '<'){
+        attrs = cbxmlattrs(elem);
+        name = cbmapget(attrs, "", -1, NULL);
+        if(!strcmp(name, "response") && elem[1] != '/'){
+          if(!flags && (tmp = cbmapget(attrs, "flags", -1, NULL)) != NULL)
+            flags = cbmemdup(tmp, -1);
+          if(!author && (tmp = cbmapget(attrs, "author", -1, NULL)) != NULL)
+            author = cbmemdup(tmp, -1);
+          if(!cdate && (tmp = cbmapget(attrs, "cdate", -1, NULL)) != NULL)
+            cdate = dateformen(tmp);
+        }
+        cbmapclose(attrs);
+      } else {
+        if(!text && strisprintable(elem)){
+          rtext = strnormalize(elem, TRUE, TRUE);
+          text = cbxmlunescape(rtext);
+          free(rtext);
+        }
+      }
     }
-    xsprintf(buf, "\n");
     cblistclose(elems);
+    if(flags && author && cdate && text)
+      xsprintf(buf, "%s\t%s\t%s\t%s\n", flags, author, cdate, text);
+    free(text);
+    free(cdate);
+    free(author);
+    free(flags);
   }
   return cbdatumtomalloc(buf, NULL);
 }
@@ -983,6 +1029,23 @@ char *arttoatom(ARTICLE *art, const char *prefix, int bnum, int rnum){
 }
 
 
+/* Get the jet lag by minute. */
+int jetlag(int min){
+  static int lag = DEFJETLAG;
+  struct tm *tp;
+  time_t t, gt, lt;
+  if(min < DEFJETLAG) lag = min;
+  if(lag < DEFJETLAG) return lag;
+  if((t = time(NULL)) < 0) return 0;
+  if(!(tp = gmtime(&t))) return 0;
+  gt = mktime(tp);
+  if(!(tp = localtime(&t))) return 0;
+  lt = mktime(tp);
+  lag = (lt - gt) / 60;
+  return lag;
+}
+
+
 /* Create the string for a current date in internal form. */
 char *datecurrent(void){
   return datestr(time(NULL));
@@ -1008,7 +1071,7 @@ char *dateforwww(const char *date, int local){
   ts.tm_sec = atoi(cblistval(elems, 5, NULL));
   sprintf(tzone, "Z");
   if(local){
-    jl = jetlag();
+    jl = jetlag(DEFJETLAG);
     ts.tm_min += jl;
     t = mktime(&ts);
     if((tp = localtime(&t)) != NULL) ts = *tp;
@@ -1046,7 +1109,7 @@ char *dateforhttp(const char *date, int local){
   ts.tm_sec = atoi(cblistval(elems, 5, NULL));
   sprintf(tzone, "GMT");
   if(local){
-    jl = jetlag();
+    jl = jetlag(DEFJETLAG);
     ts.tm_min += jl;
     t = mktime(&ts);
     if((tp = localtime(&t)) != NULL) ts = *tp;
@@ -1179,6 +1242,36 @@ void xsprintf(CBDATUM *buf, const char *format, ...){
 }
 
 
+/* Get a DES hash string of a key string. */
+const char *makecrypt(const char *key){
+  const char tbl[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789./";
+  const char *rp;
+  char salt[3];
+  int pid;
+  assert(key);
+  pid = getpid();
+  salt[0] = tbl[((pid%65536)/256)%(sizeof(tbl)-1)];
+  salt[1] = tbl[(pid%256)%(sizeof(tbl)-1)];
+  salt[2] = '\0';
+  if(!(rp = crypt(key, salt))) rp = "";
+  return rp;
+}
+
+
+/* Check whether a key matches a DES hash string. */
+int matchcrypt(const char *key, const char *code){
+  const char *rp;
+  char salt[3];
+  assert(key && code);
+  if(strlen(code) < 2) return FALSE;
+  salt[0] = code[0];
+  salt[1] = code[1];
+  salt[2] = '\0';
+  if(!(rp = crypt(key, salt))) return FALSE;
+  return strcmp(rp, code) ? FALSE : TRUE;
+}
+
+
 
 /*************************************************************************************************
  * private objects
@@ -1213,12 +1306,25 @@ static char *uniqueid(void){
     }
     fclose(ifp);
   }
-  sprintf(str, "RA%010d%05d%05d",
-          (int)time(NULL), (int)getpid() % 100000, (unsigned int)rnd % 100000);
+  sprintf(str, "%s%010d%05d%05d",
+          IDPREFIX, (int)time(NULL), (int)getpid() % 100000, (unsigned int)rnd % 100000);
   return cbmemdup(str, -1);
 }
 
 
+/* Check whether an ID string is valid.
+   The return value is true if an ID string is valid, else, it is not. */
+static int validid(const char *id){
+  int i;
+  assert(id);
+  if(!cbstrfwmatch(id, IDPREFIX)) return FALSE;
+  for(i = strlen(IDPREFIX); id[i] != '\0'; i++){
+    if(id[i] < '0' || id[i] > '9') return FALSE;
+  }
+  return TRUE;
+}
+
+
 /* Create the string for a date.
    `t' specifies the UNIX time.
    The return value is the string of the date in the internal form (YYYY-MM-DD hh:mm:ss). */
@@ -1271,23 +1377,6 @@ static int validdate(const char *date){
 }
 
 
-/* Get the jet lag in minutes.
-   The return value is the jet lag in minutes. */
-static int jetlag(void){
-  static int lag = 60 * 24;
-  struct tm *tp;
-  time_t t, gt, lt;
-  if(lag < 60 * 24) return lag;
-  if((t = time(NULL)) < 0) return 0;
-  if(!(tp = gmtime(&t))) return 0;
-  gt = mktime(tp);
-  if(!(tp = localtime(&t))) return 0;
-  lt = mktime(tp);
-  lag = (lt - gt) / 60;
-  return lag;
-}
-
-
 /* Get the name of a week day.
    `wday' specifies the number of a week day (0-6).
    The return value is the strint of the name of the week day. */
