@@ -1,7 +1,7 @@
 /*************************************************************************************************
  * Interface of common features
  *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
- * This file is part of RBBS, a personal full-text search system.
+ * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
  * License, or any later version.
@@ -66,6 +66,8 @@
 #define MDATEKEY       "mdate"           /* key of the modification date */
 #define PATHBUFSIZ     2048              /* size of a path buffer */
 #define NUMBUFSIZ      32                /* size of a number buffer */
+#define DEFJETLAG      1440              /* default value of the jet lag */
+#define IDPREFIX       "RA"              /* prefix of article ID */
 #define HASHNUM        7                 /* number of hash classes */
 #define LONESTR        "lone"            /* string for lone articles */
 #define LONECHAR       'l'               /* character for lone articles */
@@ -92,6 +94,7 @@ typedef struct {                         /* type of structure for a database han
 
 typedef struct {                         /* type of structure for an article */
   char *id;                              /* identification */
+  char *password;                        /* password for editing */
   char *flags;                           /* flags for multi purpose */
   char *language;                        /* language */
   char *category;                        /* category */
@@ -102,6 +105,7 @@ typedef struct {                         /* type of structure for an article */
   CBLIST *body;                          /* list of blocks in the body */
   CBLIST *tail;                          /* list of responses in the body */
   char *errmsg;                          /* error message */
+  char *hash;                            /* hash code of XML */
 } ARTICLE;
 
 enum {                                   /* enumeration for sorting orders */
@@ -226,15 +230,27 @@ int articleisok(ARTICLE *art);
 const char *artemsg(ARTICLE *art);
 
 
+/* Get the hash code of an article.
+   `art' specifies a handle of an article object.
+   The return value is the hash code of the article. */
+const char *arthash(ARTICLE *art);
+
+
 /* Get the ID string of an article.
    `art' specifies a handle of an article object.
    The return value is the ID string of the article. */
 const char *artid(ARTICLE *art);
 
 
+/* Get the password of an article.
+   `art' specifies a handle of an article object.
+   The return value is the password of the article. */
+const char *artpassword(ARTICLE *art);
+
+
 /* Get the flags of an article.
    `art' specifies a handle of an article object.
-   The return value is the flagst of the article. */
+   The return value is the flags of the article. */
 const char *artflags(ARTICLE *art);
 
 
@@ -332,6 +348,12 @@ char *arttohtml(ARTICLE *art, int bnum, int rnum);
 char *arttoatom(ARTICLE *art, const char *prefix, int bnum, int rnum);
 
 
+/* Get the jet lag by minute.
+   `min' specifies the jet lag to set.  If it is equal to or more than 1440, it is not set.
+   The return value is the jet lag in minutes.  The default is the jet lag of the local time. */
+int jetlag(int min);
+
+
 /* Create the string for a current date in internal form.
    `date' specifies the string of a date in internal form.
    The return value is the string of the current date in internal form. */
@@ -373,6 +395,19 @@ char *strnormalize(const char *str, int nolf, int trim);
 void xsprintf(CBDATUM *buf, const char *format, ...);
 
 
+/* Get a DES hash string of a key string.
+   `key' specifies a password to be encrypted.
+   The return value is a DES hash string of a key string. */
+const char *makecrypt(const char *key);
+
+
+/* Check whether a key matches a DES hash string.
+   `key' specifies a password to be checked.
+   `code' specifies the installed password.
+   The return value is true if a key matches a DES hash string, else it is false. */
+int matchcrypt(const char *key, const char *code);
+
+
 
 #endif                                   /* duplication check */
 
