@@ -1,7 +1,7 @@
 /*************************************************************************************************
  * Administration Utility for RBBS
  *                                                      Copyright (C) 2003-2004 Mikio Hirabayashi
- * This file is part of RBBS, a personal full-text search system.
+ * This file is part of RBBS, a web-based bulletin board system.
  * RBBS is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
  * License, or any later version.
@@ -18,6 +18,7 @@
 
 #define HTMLTYPE    "text/html"          /* content-type of HTML */
 #define ENCODING    "UTF-8"              /* encoding of the page */
+#define LISTQUERY   "action=1"           /* query to get the list */
 
 enum {                                   /* enumeration for the format of output data */
   FTXML,                                 /* original XML */
@@ -30,11 +31,11 @@ enum {                                   /* enumeration for the format of output
 /* global variables */
 const char *g_progname = NULL;           /* program name */
 const char *g_rbbsuri = NULL;            /* base URI of the RBBS system */
-const char *g_artprfx = NULL;            /* prefix of articles in HTML format */
 const char *g_dumplang = NULL;           /* language of the dumped data */
 const char *g_dumptitle = NULL;          /* title of the dump data */
 const char *g_dumpauthor = NULL;         /* author of the dump data */
 const char *g_dumpsummary = NULL;        /* summary of the dump data */
+const char *g_artprfx = NULL;            /* prefix of articles in HTML format */
 
 
 /* function prototypes */
@@ -47,14 +48,16 @@ int rundelete(int argc, char **argv);
 int runlist(int argc, char **argv);
 int rundump(int argc, char **argv);
 int runoptimize(int argc, char **argv);
-int runinit(int argc, char **argv);
+int runinitialize(int argc, char **argv);
+int runpassword(int argc, char **argv);
 int procimport(const char *name, const char *file, const char *id);
 int procexport(const char *name, const char *id, int ftype);
 int procdelete(const char *name, const char *id);
 int proclist(const char *name, int sort);
 int procdump(const char *name, int ftype);
 int procoptimize(const char *name);
-int procinit(const char *name);
+int procinitialize(const char *name);
+int procpassword(const char *key, const char *code);
 char *readfiledata(const char *file);
 void xprintf(const char *format, ...);
 
@@ -79,8 +82,10 @@ int main(int argc, char **argv){
     rv = rundump(argc, argv);
   } else if(!strcmp(argv[1], "optimize")){
     rv = runoptimize(argc, argv);
-  } else if(!strcmp(argv[1], "init")){
-    rv = runinit(argc, argv);
+  } else if(!strcmp(argv[1], "initialize")){
+    rv = runinitialize(argc, argv);
+  } else if(!strcmp(argv[1], "password")){
+    rv = runpassword(argc, argv);
   } else {
     usage();
   }
@@ -91,18 +96,21 @@ int main(int argc, char **argv){
 /* set global variables */
 void setglobal(void){
   const char *tmp;
+  char *buf;
   g_rbbsuri = "http://localhost/rbbs/rbbs.cgi";
-  g_artprfx = "http://localhost/rbbs/rbbs.cgi?id=";
   g_dumplang = "en";
   g_dumptitle = "RBBS Articles";
   g_dumpauthor = "anonymous";
   g_dumpsummary = "This page includes all articles of the Raving Bulletin Board System.";
   if((tmp = getenv("RBBSURI")) != NULL) g_rbbsuri = tmp;
-  if((tmp = getenv("RBBSARTPRFX")) != NULL) g_artprfx = tmp;
+  if((tmp = getenv("RBBSJETLAG")) != NULL) jetlag(atoi(tmp));
   if((tmp = getenv("RBBSDUMPLANG")) != NULL) g_dumplang = tmp;
   if((tmp = getenv("RBBSDUMPTITLE")) != NULL) g_dumptitle = tmp;
   if((tmp = getenv("RBBSDUMPAUTHOR")) != NULL) g_dumpauthor = tmp;
   if((tmp = getenv("RBBSDUMPSUMMARY")) != NULL) g_dumpsummary = tmp;
+  buf = cbsprintf("%s?id=", g_rbbsuri);
+  cbglobalgc(buf, free);
+  g_artprfx = buf;
 }
 
 
@@ -117,7 +125,8 @@ void usage(void){
   fprintf(stderr, "  %s list [-scd] [-sma] [-smd] name\n", g_progname);
   fprintf(stderr, "  %s dump [-ft] [-fh] [-fa] name\n", g_progname);
   fprintf(stderr, "  %s optimize name\n", g_progname);
-  fprintf(stderr, "  %s init name\n", g_progname);
+  fprintf(stderr, "  %s initialize name\n", g_progname);
+  fprintf(stderr, "  %s password key [code]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -285,8 +294,8 @@ int runoptimize(int argc, char **argv){
 }
 
 
-/* parse arguments of init command */
-int runinit(int argc, char **argv){
+/* parse arguments of initialize command */
+int runinitialize(int argc, char **argv){
   char *name;
   int i, rv;
   name = NULL;
@@ -300,7 +309,28 @@ int runinit(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = procinit(name);
+  rv = procinitialize(name);
+  return rv;
+}
+
+
+/* parse arguments of password command */
+int runpassword(int argc, char **argv){
+  char *key, *code;
+  int i, rv;
+  key = NULL;
+  code = NULL;
+  for(i = 2; i < argc; i++){
+    if(!key){
+      key = argv[i];
+    } else if(!code){
+      code = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!key) usage();
+  rv = procpassword(key, code);
   return rv;
 }
 
@@ -483,7 +513,7 @@ int procdump(const char *name, int ftype){
               HTMLTYPE, ENCODING);
       xprintf("<meta name=\"author\" content=\"%@\" />\n", g_dumpauthor);
       xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
-      xprintf("<link rel=\"contents\" href=\"%@\" />\n", g_rbbsuri);
+      xprintf("<link rel=\"contents\" href=\"%@?%s\" />\n", g_rbbsuri, LISTQUERY);
       xprintf("</head>\n");
       xprintf("<body>\n");
       xprintf("<h1>%@</h1>\n", g_dumptitle);
@@ -593,7 +623,7 @@ int procoptimize(const char *name){
 
 
 /* remove a database */
-int procinit(const char *name){
+int procinitialize(const char *name){
   RBBS *rbbs;
   int rv;
   rv = 0;
@@ -613,6 +643,22 @@ int procinit(const char *name){
 }
 
 
+/* create a password or check a password */
+int procpassword(const char *key, const char *code){
+  if(code){
+    if(matchcrypt(key, code)){
+      printf("%s: ok\n", g_progname);
+    } else {
+      fprintf(stderr, "%s: unmatched\n", g_progname);
+      return 1;
+    }
+  } else {
+    printf("%s\n", makecrypt(key));
+  }
+  return 0;
+}
+
+
 /* read whole data of a file */
 char *readfiledata(const char *file){
   CBDATUM *buf;
